@@ -33,9 +33,10 @@ class ProteinManager:
         else:
             self.error_status = False
             
-    # decorators 
-    # To be added in 0.1.6 to streamline ID
-    # checking
+
+    #----------------------------------------------------------------------------
+    # decorators for streamlining ID checking
+   
     def convertIDToUpperCase(fn):
         def wrapped(_, a):
             return fn(_, str(a).upper())
@@ -45,7 +46,36 @@ class ProteinManager:
         def wrapped(_, a):
             return fn(_, [str(x).upper() for x in a])
         return wrapped
+    #----------------------------------------------------------------------------
+
+    
+    # Firstly we have the general pm functions (i.e. those not linked to
+    # a specific accession
+    
             
+  
+    def keys(self):
+        """ Return a list of the keys in the datastore (note this hides 
+        the always present '-1' key
+        """
+        return self.datastore.keys()
+
+
+    def accession_classes(self):
+        """ Return a list defines the names of the possible
+            'other_accession' accession types. 
+
+            Useful for filtering out non-types by doing
+
+               for i in other_accessions_list:
+                  if i in pm.accession_classes():
+                      some functionalty 
+                      
+        """
+
+        return(self.datastore.accession_classes())
+
+
     @convertIDToUpperCase
     def has_key(self, ID):
         """ Check if the datastore has a protein ID loaded or not. Does
@@ -53,12 +83,6 @@ class ProteinManager:
         """
         return self.datastore.has_key(ID)
 
-    def keys(self):
-        """ Return a list of the keys in the datastore (note this hides 
-        the always present '-1' key
-        """
-        return self.datastore.keys()
-
     @convertIDToUpperCase
     def error(self, ID):
         """ Determine if the record associated with the ID had an error 
@@ -182,6 +206,16 @@ class ProteinManager:
         """
         return self.datastore.get_species(ID)
 
+    @convertIDToUpperCase
+    def get_host(self, ID):
+        """ Returns the host of the virus if the ID corresponds to a 
+            viral protein in the format <scientific name> (<common name>),
+            e.g. Homo Sapiens (Human).
+            
+            If not a viral protein returns the string 'N/A'
+        """
+        return self.datastore.get_host(ID)
+
     @convertIDToUpperCase
     def get_other_accessions(self, ID):
         """ Returns a list of tuples which define ("type of accession", "value") 
@@ -268,5 +302,8 @@ class ProteinManager:
         """ Get the number of items in the internal datastore """
         return self.datastore.datastore_size()
 
+    def load_local_uniprot_xml(self, filename):
+        self.datastore.loadfile(filename)
+
     
     
